@@ -1,4 +1,4 @@
-/*	$OpenBSD: ca.c,v 1.87 2021/12/14 13:44:36 tobhe Exp $	*/
+/*	$OpenBSD: ca.c,v 1.89 2022/11/07 22:39:52 tobhe Exp $	*/
 
 /*
  * Copyright (c) 2020-2021 Tobias Heider <tobhe@openbsd.org>
@@ -42,6 +42,11 @@
 #include <openssl/evp.h>
 #include <openssl/sha.h>
 #include <openssl/rsa.h>
+#include <openssl/opensslv.h>
+
+#if OPENSSL_VERSION_NUMBER >= 0x30000000
+#include <openssl/provider.h>
+#endif
 
 #include "iked.h"
 #include "ikev2.h"
@@ -329,6 +334,32 @@ ca_setcert(struct iked *env, struct iked_sahdr *sh, struct iked_id *id,
 	return (0);
 }
 
+static int
+ca_setscert(struct iked *env, struct iked_sahdr *sh, uint8_t type, X509 *cert)
+{
+	struct iovec		iov[3];
+	int			iovcnt = 0;
+	struct ibuf		*buf;
+	int			ret;
+
+	if ((buf = ca_x509_serialize(cert)) == NULL)
+		return (-1);
+
+	iov[iovcnt].iov_base = sh;
+	iov[iovcnt].iov_len = sizeof(*sh);
+	iovcnt++;
+	iov[iovcnt].iov_base = &type;
+	iov[iovcnt].iov_len = sizeof(type);
+	iovcnt++;
+	iov[iovcnt].iov_base = ibuf_data(buf);
+	iov[iovcnt].iov_len = ibuf_size(buf);
+	iovcnt++;
+
+	ret = proc_composev(&env->sc_ps, PROC_IKEV2, IMSG_SCERT, iov, iovcnt);
+	ibuf_release(buf);
+	return (ret);
+}
+
 int
 ca_setreq(struct iked *env, struct iked_sa *sa,
     struct iked_static_id *localid, uint8_t type, uint8_t more, uint8_t *data,
@@ -542,6 +573,51 @@ ca_getcert(struct iked *env, struct imsg *imsg)
 	return (0);
 }
 
+static unsigned int
+ca_chain_by_issuer(struct ca_store *store, X509_NAME *subject,
+    struct iked_static_id *id, X509 **dst, size_t dstlen)
+{
+	STACK_OF(X509_OBJECT)	*h;
+	X509_OBJECT		*xo;
+	X509			*cert;
+	int			i;
+	unsigned int		n;
+	X509_NAME		*issuer, *subj;
+
+        if (subject == NULL || dstlen == 0)
+		return (0);
+
+	if ((cert = ca_by_issuer(store->ca_certs, subject, id)) != NULL) {
+		*dst = cert;
+		return (1);
+	}
+
+	h = X509_STORE_get0_objects(store->ca_cas);
+	for (i = 0; i < sk_X509_OBJECT_num(h); i++) {
+		xo = sk_X509_OBJECT_value(h, i);
+		if (X509_OBJECT_get_type(xo) != X509_LU_X509)
+			continue;
+		cert = X509_OBJECT_get0_X509(xo);
+		if ((issuer = X509_get_issuer_name(cert)) == NULL)
+			continue;
+		if (X509_NAME_cmp(subject, issuer) == 0) {
+			if ((subj = X509_get_subject_name(cert)) == NULL)
+				continue;
+			/* Skip root CAs */
+			if (X509_NAME_cmp(subj, issuer) == 0)
+				continue;
+			n = ca_chain_by_issuer(store, subj, id,
+			    dst + 1, dstlen - 1);
+			if (n > 0) {
+				*dst = cert;
+				return (n + 1);
+			}
+		}
+	}
+
+	return (0);
+}
+
 int
 ca_getreq(struct iked *env, struct imsg *imsg)
 {
@@ -552,6 +628,8 @@ ca_getreq(struct iked *env, struct imsg *imsg)
 	size_t			 len;
 	unsigned int		 i;
 	X509			*ca = NULL, *cert = NULL;
+	X509			*chain[IKED_SCERT_MAX + 1];
+	size_t			 chain_len = 0;
 	struct ibuf		*buf;
 	struct iked_static_id	 id;
 	char			 idstr[IKED_ID_SIZE];
@@ -611,10 +689,12 @@ ca_getreq(struct iked *env, struct imsg *imsg)
 			if (subj_name == NULL)
 				return (-1);
 			log_debug("%s: found CA %s", __func__, subj_name);
-			free(subj_name);
+			OPENSSL_free(subj_name);
 
-			if ((cert = ca_by_issuer(store->ca_certs,
-			    subj, &id)) != NULL) {
+			chain_len = ca_chain_by_issuer(store, subj, &id,
+			    chain, nitems(chain));
+			if (chain_len > 0) {
+				cert = chain[chain_len - 1];
 				if (!ca_cert_local(env, cert)) {
 					log_info("%s: found cert with matching "
 					    "ID but without matching key.",
@@ -624,6 +704,10 @@ ca_getreq(struct iked *env, struct imsg *imsg)
 				break;
 			}
 		}
+
+		for (i = chain_len; i >= 2; i--)
+			ca_setscert(env, &sh, type, chain[i - 2]);
+
 		/* Fallthrough */
 	case IKEV2_CERT_NONE:
  fallback:
@@ -668,7 +752,7 @@ ca_getreq(struct iked *env, struct imsg *imsg)
 			return (-1);
 		log_debug("%s: found local certificate %s", __func__,
 		    subj_name);
-		free(subj_name);
+		OPENSSL_free(subj_name);
 
 		if ((buf = ca_x509_serialize(cert)) == NULL)
 			return (-1);
@@ -843,7 +927,7 @@ ca_reload(struct iked *env)
 		if (subj_name == NULL)
 			return (-1);
 		log_debug("%s: %s", __func__, subj_name);
-		free(subj_name);
+		OPENSSL_free(subj_name);
 
 		if (ibuf_add(env->sc_certreq, md, len) != 0) {
 			ibuf_release(env->sc_certreq);
@@ -1117,10 +1201,10 @@ ca_subjectpubkey_digest(X509 *x509, uint8_t *md, unsigned int *size)
 	if (buflen == 0)
 		return (-1);
 	if (!EVP_Digest(buf, buflen, md, size, EVP_sha1(), NULL)) {
-		free(buf);
+		OPENSSL_free(buf);
 		return (-1);
 	}
-	free(buf);
+	OPENSSL_free(buf);
 
 	return (0);
 }
@@ -1147,7 +1231,7 @@ ca_store_info(struct iked *env, const char *msg, X509_STORE *ctx)
 		    (name = X509_NAME_oneline(subject, NULL, 0)) == NULL)
 			continue;
 		buflen = asprintf(&buf, "%s: %s\n", msg, name);
-		free(name);
+		OPENSSL_free(name);
 		if (buflen == -1)
 			continue;
 		proc_compose(&env->sc_ps, PROC_CONTROL, IMSG_CTL_SHOW_CERTSTORE,
@@ -1400,6 +1484,10 @@ ca_privkey_to_method(struct iked_id *privkey)
 	return (method);
 }
 
+/*
+ * Return dynamically allocated buffer containing certificate name.
+ * The resulting buffer must be freed with OpenSSL_free().
+ */
 char *
 ca_asn1_name(uint8_t *asn1, size_t len)
 {
@@ -1717,7 +1805,7 @@ ca_validate_cert(struct iked *env, struct iked_static_id *id,
 		if (subj_name == NULL)
 			goto err;
 		log_debug("%s: %s %.100s", __func__, subj_name, errstr);
-		free(subj_name);
+		OPENSSL_free(subj_name);
 	}
  err:
 
@@ -1881,12 +1969,28 @@ ca_x509_subjectaltname_get(X509 *cert, struct iked_id *retid)
 void
 ca_sslinit(void)
 {
+#if OPENSSL_VERSION_NUMBER >= 0x30000000
+	/*
+	 * This provider contains all the regular and safe algorithms,
+	 * such as AES, SHA...
+	 */
+	if (OSSL_PROVIDER_load(NULL, "default") == NULL)
+		fatal("error enabling default provider");
+
+	/*
+	 * This provider contains unsafe or obsolete, such as MD4 which
+	 * we need for MSCHAPv2
+	 */
+	if (OSSL_PROVIDER_load(NULL, "legacy") == NULL)
+		fatal("error enabling legacy provider");
+#else
 	OpenSSL_add_all_algorithms();
 	ERR_load_crypto_strings();
 
 	/* Init hardware crypto engines. */
 	ENGINE_load_builtin_engines();
 	ENGINE_register_all_complete();
+#endif
 }
 
 void
