@@ -1,4 +1,4 @@
-/*	$OpenBSD: config.c,v 1.85 2022/05/08 20:26:31 tobhe Exp $	*/
+/*	$OpenBSD: config.c,v 1.90 2022/11/18 18:56:39 mbuhl Exp $	*/
 
 /*
  * Copyright (c) 2019-2021 Tobias Heider <tobhe@openbsd.org>
@@ -64,6 +64,7 @@ config_new_sa(struct iked *env, int initiator)
 	gettimeofday(&sa->sa_timecreated, NULL);
 	memcpy(&sa->sa_timeused, &sa->sa_timecreated, sizeof(sa->sa_timeused));
 
+	ikestat_inc(env, ikes_sa_created);
 	return (sa);
 }
 
@@ -114,6 +115,8 @@ config_free_fragments(struct iked_frag *frag)
 void
 config_free_sa(struct iked *env, struct iked_sa *sa)
 {
+	int i;
+
 	timer_del(env, &sa->sa_timer);
 	timer_del(env, &sa->sa_keepalive);
 	timer_del(env, &sa->sa_rekey);
@@ -169,6 +172,8 @@ config_free_sa(struct iked *env, struct iked_sa *sa)
 	ibuf_release(sa->sa_rid.id_buf);
 	ibuf_release(sa->sa_icert.id_buf);
 	ibuf_release(sa->sa_rcert.id_buf);
+	for (i = 0; i < IKED_SCERT_MAX; i++)
+		ibuf_release(sa->sa_scert[i].id_buf);
 	ibuf_release(sa->sa_localauth.id_buf);
 	ibuf_release(sa->sa_peerauth.id_buf);
 
@@ -181,6 +186,11 @@ config_free_sa(struct iked *env, struct iked_sa *sa)
 	free(sa->sa_cp_dns);
 
 	free(sa->sa_tag);
+
+	if (sa->sa_state == IKEV2_STATE_ESTABLISHED)
+		ikestat_dec(env, ikes_sa_established_current);
+	ikestat_inc(env, ikes_sa_removed);
+
 	free(sa);
 }
 
@@ -211,10 +221,15 @@ config_free_policy(struct iked *env, struct iked_policy *pol)
 	if (pol->pol_flags & IKED_POLICY_REFCNT)
 		goto remove;
 
+	/*
+	 * Remove policy from the sc_policies list, but increment
+	 * refcount for every SA linked for the policy.
+	 */
+	pol->pol_flags |= IKED_POLICY_REFCNT;
+
 	TAILQ_REMOVE(&env->sc_policies, pol, pol_entry);
 
 	TAILQ_FOREACH(sa, &pol->pol_sapeers, sa_peer_entry) {
-		/* Remove from the policy list, but keep for existing SAs */
 		if (sa->sa_policy == pol)
 			policy_ref(env, pol);
 		else
@@ -333,6 +348,7 @@ config_free_childsas(struct iked *env, struct iked_childsas *head,
 			childsa_free(ipcomp);
 		}
 		childsa_free(csa);
+		ikestat_inc(env, ikes_csa_removed);
 	}
 }
 
