@@ -1,4 +1,4 @@
-/*	$OpenBSD: iked.c,v 1.58 2021/09/01 15:30:06 tobhe Exp $	*/
+/*	$OpenBSD: iked.c,v 1.62 2021/12/01 16:42:12 deraadt Exp $	*/
 
 /*
  * Copyright (c) 2019 Tobias Heider <tobias.heider@stusta.de>
@@ -40,6 +40,7 @@
 
 #include "iked.h"
 #include "ikev2.h"
+#include "version.h"
 
 __dead void usage(void);
 
@@ -66,7 +67,7 @@ usage(void)
 {
 	extern char	*__progname;
 
-	fprintf(stderr, "usage: %s [-dnSTtv] [-D macro=value] "
+	fprintf(stderr, "usage: %s [-dnSTtVv] [-D macro=value] "
 	    "[-f file] [-p udpencap_port] [-s socket]\n", __progname);
 	exit(1);
 }
@@ -102,7 +103,7 @@ main(int argc, char *argv[])
 	argv = saved_av;
 #endif
 
-	while ((c = getopt(argc, argv, "6D:df:np:Ss:Ttv")) != -1) {
+	while ((c = getopt(argc, argv, "6D:df:np:Ss:TtvV")) != -1) {
 		switch (c) {
 		case '6':
 			log_warnx("the -6 option is ignored and will be "
@@ -151,6 +152,9 @@ main(int argc, char *argv[])
 			verbose++;
 			opts |= IKED_OPT_VERBOSE;
 			break;
+		case 'V':
+			fprintf(stderr, "OpenIKED %s\n", IKED_VERSION);
+			return 0;
 		default:
 			usage();
 		}
@@ -223,7 +227,7 @@ main(int argc, char *argv[])
 
 	proc_listen(ps, procs, nitems(procs));
 
-#if defined(HAVE_VROUTE) || defined(HAVE_VROUTE_NETLINK)
+#if defined(HAVE_VROUTE)
 	vroute_init(env);
 #endif
 
@@ -343,7 +347,7 @@ parent_reload(struct iked *env, int reset, const char *filename)
 		config_setcoupled(env, env->sc_decoupled ? 0 : 1);
 		config_setocsp(env);
 		config_setcertpartialchain(env);
- 		/* Must be last */
+		/* Must be last */
 		config_setmode(env, env->sc_passive ? 1 : 0);
 	} else {
 		config_setreset(env, reset, PROC_IKEV2);
@@ -381,7 +385,7 @@ parent_sig_handler(int sig, short event, void *arg)
 		/* FALLTHROUGH */
 	case SIGCHLD:
 		do {
-			int len;
+			int len = 0;
 
 			pid = waitpid(-1, &status, WNOHANG);
 			if (pid <= 0)
@@ -397,8 +401,10 @@ parent_sig_handler(int sig, short event, void *arg)
 					fail = 1;
 					len = asprintf(&cause,
 					    "exited abnormally");
-				} else
+				} else {
 					len = asprintf(&cause, "exited okay");
+					break;
+				}
 			} else
 				fatalx("unexpected cause of SIGCHLD");
 
@@ -487,7 +493,7 @@ parent_dispatch_ikev2(int fd, struct privsep_proc *p, struct imsg *imsg)
 	struct iked	*env = p->p_ps->ps_env;
 
 	switch (imsg->hdr.type) {
-#if defined(HAVE_VROUTE) || defined(HAVE_VROUTE_NETLINK)
+#if defined(HAVE_VROUTE)
 	case IMSG_IF_ADDADDR:
 	case IMSG_IF_DELADDR:
 		return (vroute_getaddr(env, imsg));
@@ -512,7 +518,7 @@ parent_shutdown(struct iked *env)
 {
 	proc_kill(&env->sc_ps);
 
-#if defined(HAVE_VROUTE) || defined(HAVE_VROUTE_NETLINK)
+#if defined(HAVE_VROUTE)
 	vroute_cleanup(env);
 #endif
 	free(env->sc_vroute);
