@@ -1,4 +1,4 @@
-/*	$OpenBSD: ikev2_pld.c,v 1.123 2022/03/14 12:58:55 tobhe Exp $	*/
+/*	$OpenBSD: ikev2_pld.c,v 1.125 2022/09/19 20:54:02 tobhe Exp $	*/
 
 /*
  * Copyright (c) 2019 Tobias Heider <tobias.heider@stusta.de>
@@ -827,9 +827,9 @@ ikev2_pld_cert(struct iked *env, struct ikev2_payload *pld,
 
 	certid = &msg->msg_parent->msg_cert;
 	if (certid->id_type) {
-		log_info("%s: multiple cert payloads not supported",
+		log_debug("%s: multiple cert payloads, ignoring",
 		   SPI_SA(sa, __func__));
-		return (-1);
+		return (0);
 	}
 
 	if ((certid->id_buf = ibuf_new(buf, len)) == NULL) {
@@ -1609,6 +1609,7 @@ ikev2_pld_ef(struct iked *env, struct ikev2_payload *pld,
 	size_t				 frag_num, frag_total;
 	size_t				 len;
 	int				 ret = -1;
+	int				 processed = 0;
 	ssize_t				 elen;
 
 	buf = msgbuf + offset;
@@ -1620,6 +1621,8 @@ ikev2_pld_ef(struct iked *env, struct ikev2_payload *pld,
 	buf = msgbuf + offset;
 	len = left - sizeof(frag);
 
+	ikestat_inc(env, ikes_frag_rcvd);
+
 	/* Limit number of total fragments to avoid DOS */
 	if (frag_total > IKED_FRAG_TOTAL_MAX ) {
 		log_debug("%s: Total Fragments too big  %zu",
@@ -1705,10 +1708,15 @@ ikev2_pld_ef(struct iked *env, struct ikev2_payload *pld,
 	} else {
 		ret = 0;
 	}
+	processed = 1;
+
 done:
+	if (!processed)
+		ikestat_inc(env, ikes_frag_rcvd_drop);
 	ibuf_release(e);
 	return (ret);
 dropall:
+	ikestat_add(env, ikes_frag_rcvd_drop, sa_frag->frag_count + 1);
 	config_free_fragments(sa_frag);
 	ibuf_release(e);
 	return -1;
@@ -1726,6 +1734,7 @@ ikev2_frags_reassemble(struct iked *env, struct ikev2_payload *pld,
 	size_t				 i;
 	struct iked_message		 emsg;
 	int				 ret = -1;
+	int				 processed = 0;
 
 	/* Reassemble fragments to single buffer */
 	if ((e = ibuf_new(NULL, sa_frag->frag_total_size)) == NULL) {
@@ -1769,7 +1778,12 @@ ikev2_frags_reassemble(struct iked *env, struct ikev2_payload *pld,
 
 	ret = ikev2_pld_payloads(env, &emsg, 0, ibuf_size(e),
 	    sa_frag->frag_nextpayload);
+	processed = 1;
 done:
+	if (processed)
+		ikestat_add(env, ikes_frag_reass_ok, sa_frag->frag_total);
+	else
+		ikestat_add(env, ikes_frag_reass_drop, sa_frag->frag_total);
 	config_free_fragments(sa_frag);
 	ibuf_release(e);
 
