@@ -1,4 +1,4 @@
-/*	$OpenBSD: pfkey.c,v 1.80 2021/11/25 19:41:03 tobhe Exp $	*/
+/*	$OpenBSD: pfkey.c,v 1.81 2022/07/22 15:33:53 tobhe Exp $	*/
 
 /*
  * Copyright (c) 2020-2021 Tobias Heider <tobhe@openbsd.org>
@@ -863,16 +863,22 @@ pfkey_sa(struct iked *env, uint8_t satype, uint8_t action, struct iked_childsa *
 		nat_type.sadb_x_nat_t_type_len = sizeof(nat_type) / 8;
 		nat_type.sadb_x_nat_t_type_exttype = SADB_X_EXT_NAT_T_TYPE;
 		nat_type.sadb_x_nat_t_type_type = UDP_ENCAP_ESPINUDP;
+
 		bzero(&nat_sport, sizeof(nat_sport));
 		nat_sport.sadb_x_nat_t_port_len = sizeof(nat_sport) / 8;
 		nat_sport.sadb_x_nat_t_port_exttype = SADB_X_EXT_NAT_T_SPORT;
 		nat_sport.sadb_x_nat_t_port_port =
-		    sa->csa_ikesa->sa_local.addr_port;
+		    sa->csa_dir == IPSP_DIRECTION_OUT ?
+		    sa->csa_ikesa->sa_local.addr_port :
+		    sa->csa_ikesa->sa_peer.addr_port;
+
 		bzero(&nat_dport, sizeof(nat_dport));
 		nat_dport.sadb_x_nat_t_port_len = sizeof(nat_dport) / 8;
 		nat_dport.sadb_x_nat_t_port_exttype = SADB_X_EXT_NAT_T_DPORT;
 		nat_dport.sadb_x_nat_t_port_port =
-		    sa->csa_ikesa->sa_peer.addr_port;
+		    sa->csa_dir == IPSP_DIRECTION_OUT ?
+		    sa->csa_ikesa->sa_peer.addr_port :
+		    sa->csa_ikesa->sa_local.addr_port;
 
 		log_debug("%s: NAT-T: type=%s (%d) sport=%d dport=%d",
 		    __func__,
@@ -2281,13 +2287,13 @@ pfkey_process(struct iked *env, struct pfkey_message *pm)
 		flow.flow_src.addr_port = htons(socket_getport(ssrc));
 		if ((slen = ssrc->sa_len) > sizeof(flow.flow_src.addr)) {
 			log_debug("%s: invalid src address len", __func__);
-			return (0);
+			goto out;
 		}
 		memcpy(&flow.flow_src.addr, ssrc, slen);
 		if (socket_af((struct sockaddr *)&flow.flow_src.addr,
 		    flow.flow_src.addr_port) == -1) {
 			log_debug("%s: invalid address", __func__);
-			return (0);
+			goto out;
 		}
 
 		if ((sa_addr = pfkey_find_ext(reply, rlen,
@@ -2300,13 +2306,13 @@ pfkey_process(struct iked *env, struct pfkey_message *pm)
 		flow.flow_dst.addr_port = htons(socket_getport(sdst));
 		if ((slen = sdst->sa_len) > sizeof(flow.flow_dst.addr)) {
 			log_debug("%s: invalid dst address len", __func__);
-			return (0);
+			goto out;
 		}
 		memcpy(&flow.flow_dst.addr, sdst, slen);
 		if (socket_af((struct sockaddr *)&flow.flow_dst.addr,
 		    flow.flow_dst.addr_port) == -1) {
 			log_debug("%s: invalid address", __func__);
-			return (0);
+			goto out;
 		}
 
 		if ((sa_addr = pfkey_find_ext(reply, rlen,
@@ -2330,8 +2336,7 @@ pfkey_process(struct iked *env, struct pfkey_message *pm)
 			break;
 		default:
 			log_debug("%s: bad address family", __func__);
-			free(reply);
-			return (0);
+			goto out;
 		}
 
 		if ((sa_addr = pfkey_find_ext(reply, rlen,
@@ -2355,8 +2360,7 @@ pfkey_process(struct iked *env, struct pfkey_message *pm)
 			break;
 		default:
 			log_debug("%s: bad address family", __func__);
-			free(reply);
-			return (0);
+			goto out;
 		}
 
 		switch (hdr->sadb_msg_satype) {
