@@ -1,4 +1,4 @@
-/*	$OpenBSD: types.h,v 1.48 2022/04/13 20:54:55 deraadt Exp $	*/
+/*	$OpenBSD: types.h,v 1.51 2022/09/19 20:54:02 tobhe Exp $	*/
 
 /*
  * Copyright (c) 2019 Tobias Heider <tobias.heider@stusta.de>
@@ -41,6 +41,8 @@
 #define IKED_PRIVKEY		IKED_CA "private/local.key"
 #define IKED_PUBKEY		"local.pub"
 
+#define IKED_VENDOR_ID		"OpenIKED-"
+
 #define IKED_OCSP_RESPCERT	"ocsp/responder.crt"
 
 #define IKED_OPT_VERBOSE	0x00000001
@@ -115,6 +117,7 @@ enum imsg_type {
 	IMSG_CERTVALID,
 	IMSG_CERTINVALID,
 	IMSG_CERT_PARTIAL_CHAIN,
+	IMSG_SCERT,
 	IMSG_IF_ADDADDR,
 	IMSG_IF_DELADDR,
 	IMSG_VROUTE_ADD,
@@ -127,7 +130,8 @@ enum imsg_type {
 	IMSG_AUTH,
 	IMSG_PRIVKEY,
 	IMSG_PUBKEY,
-	IMSG_CTL_SHOW_CERTSTORE
+	IMSG_CTL_SHOW_CERTSTORE,
+	IMSG_CTL_SHOW_STATS
 };
 
 enum privsep_procid {
