@@ -29,10 +29,11 @@
 
 #include <event.h>
 #include <string.h>
+#include <unistd.h>
 
 #include <iked.h>
 
-int vroute_setroute(struct iked *, uint8_t, struct sockaddr *, uint8_t,
+int vroute_setroute(struct iked *, uint32_t, struct sockaddr *, uint8_t,
     struct sockaddr *, int);
 int vroute_doroute(struct iked *, uint8_t, struct sockaddr *,
     struct sockaddr *, struct sockaddr *, int);
@@ -230,25 +231,22 @@ vroute_getaddr(struct iked *env, struct imsg *imsg)
 }
 
 int
-vroute_setdns(struct iked *env, int add, struct sockaddr *dns,
+vroute_setdns(struct iked *env, int add, struct sockaddr *addr,
     unsigned int ifidx)
 {
 	struct iovec		 iov[2];
-	int			 iovcnt = 0;
 
-	if (dns == NULL)
+	if (addr == NULL)
 		return (0);
 
-	iov[0].iov_base = &ifidx;
-	iov[0].iov_len = sizeof(ifidx);
-	iovcnt++;
+	iov[0].iov_base = addr;
+	iov[0].iov_len = SA_LEN(addr);
 
-	iov[1].iov_base = dns;
-	iov[1].iov_len = SA_LEN(dns);
-	iovcnt++;
+	iov[1].iov_base = &ifidx;
+	iov[1].iov_len = sizeof(ifidx);
 
 	return (proc_composev(&env->sc_ps, PROC_PARENT,
-	    add ? IMSG_VDNS_ADD: IMSG_VDNS_DEL, iov, iovcnt));
+	    add ? IMSG_VDNS_ADD: IMSG_VDNS_DEL, iov, 2));
 }
 
 int
@@ -257,27 +255,27 @@ vroute_getdns(struct iked *env, struct imsg *imsg)
 	struct sockaddr		*dns;
 	uint8_t			*ptr;
 	size_t			 left;
-	int			 add, ifidx;
+	int			 add;
+	unsigned int		 ifidx;
 
 	ptr = imsg->data;
 	left = IMSG_DATA_SIZE(imsg);
 
-	if (left < sizeof(ifidx))
-		fatalx("bad length imsg received");
-
-	memcpy(&ifidx, ptr, sizeof(ifidx));
-	ptr += sizeof(ifidx);
-	left -= sizeof(ifidx);
-
 	if (left < sizeof(*dns))
 		fatalx("bad length imsg received");
-	dns = (struct sockaddr *) ptr;
 
+	dns = (struct sockaddr *) ptr;
 	if (left < SA_LEN(dns))
 		fatalx("bad length imsg received");
 	ptr += SA_LEN(dns);
 	left -= SA_LEN(dns);
 
+	if (left != sizeof(ifidx))
+		fatalx("bad length imsg received");
+	memcpy(&ifidx, ptr, sizeof(ifidx));
+	ptr += sizeof(ifidx);
+	left -= sizeof(ifidx);
+
 	add = (imsg->hdr.type == IMSG_VDNS_ADD);
 	return (vroute_dodns(env, dns, add, ifidx));
 }
@@ -404,7 +402,7 @@ vroute_setdelroute(struct iked *env, uint8_t rdomain, struct sockaddr *dst,
 }
 
 int
-vroute_setroute(struct iked *env, uint8_t rdomain, struct sockaddr *dst,
+vroute_setroute(struct iked *env, uint32_t rdomain, struct sockaddr *dst,
     uint8_t mask, struct sockaddr *addr, int type)
 {
 	struct sockaddr_storage	 sa;
@@ -463,7 +461,7 @@ vroute_getroute(struct iked *env, struct imsg *imsg)
 	uint8_t			*ptr;
 	size_t			 left;
 	int			 type = 0;
-	uint8_t			 rdomain;
+	uint32_t		 rdomain;
 	int			 host = 1;
 
 	ptr = (uint8_t *)imsg->data;
@@ -528,7 +526,7 @@ vroute_getcloneroute(struct iked *env, struct imsg *imsg)
 	struct sockaddr_storage	 mask;
 	uint8_t			*ptr;
 	size_t			 left;
-	uint8_t			 rdomain;
+	uint32_t		 rdomain;
 
 	ptr = (uint8_t *)imsg->data;
 	left = IMSG_DATA_SIZE(imsg);
@@ -727,13 +725,23 @@ vroute_dodns(struct iked *env, struct sockaddr *dns, int add,
 	if (if_indextoname(ifindex, ifname) == NULL)
 		return (-1);
 
+#define resolvectl(...) do {				\
+		if (fork() == 0) {			\
+			execl("/usr/bin/resolvectl",	\
+			    __VA_ARGS__);		\
+			exit(0);			\
+		}					\
+	} while (0)
+
 	if (add && dns) {
-		/* XXX: use native dbus instead */
-		run_command("resolvectl dns %s %s", ifname,
-		    print_host(dns, NULL, 0));
-		run_command("resolvectl domain %s ~.", ifname);
+		resolvectl("/usr/bin/resolvectl", "dns", ifname,
+		    print_host(dns, NULL, 0), (char *) NULL);
+
+		resolvectl("/usr/bin/resolvectl", "domain", ifname,
+		    "~.", (char *) NULL);
 	} else {
-		run_command("resolvectl revert %s", ifname);
+		resolvectl("/usr/bin/resolvectl", "revert", ifname,
+		    (char *) NULL);
 	}
 #endif
 	return (0);
